/*
This is a part of the Mouse LiteStep module source code.

Copyright (C) 2001 Erik Christiansson

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "../lsapi/lsapi.h"

extern "C"
{
  __declspec( dllexport ) int initModuleEx(HWND parent, HINSTANCE dll, LPCTSTR szPath);
  __declspec( dllexport ) void quitModule(HINSTANCE dllInst);
}
void bangMove(HWND hCaller, LPCSTR szArgs);
void bangClick(HWND hCaller, LPCSTR szArgs);

int initModuleEx(HWND parentWnd, HINSTANCE dllInst, LPCSTR szPath)
{
	AddBangCommand("!MouseMove", bangMove);
	AddBangCommand("!MouseClick", bangClick);

	return 0;
}

void quitModule(HINSTANCE dllInst)
{
	RemoveBangCommand("!MouseMove");
	RemoveBangCommand("!MouseClick");
}

void bangMove(HWND hCaller, LPCSTR szArgs)
{
	char szX[10], szY[10];
	char* tokens[2] = {szX, szY};
	int nCount, nX, nY;

	nCount = LCTokenize(szArgs, tokens, 2, NULL);
	if(nCount == 2)
	{
		nX = atoi(szX);
		nY = atoi(szY);

		if(nX < 0) nX += GetSystemMetrics(SM_CXSCREEN);
		if(nY < 0) nY += GetSystemMetrics(SM_CYSCREEN);

		SetCursorPos(nX, nY);
	}
}

void bangClick(HWND hCaller, LPCSTR szArgs)
{
	char szToken1[10], szToken2[10], szToken3[10];
	char* tokens[3] = {szToken1, szToken2, szToken3};
	char* szButton = NULL;
	char* szX = NULL;
	char* szY = NULL;
	BOOL bMove = FALSE;
	BOOL bDouble = FALSE;
	int nNewX, nNewY, nOldX, nOldY;
	DWORD dwDown = MOUSEEVENTF_LEFTDOWN;
	DWORD dwUp = MOUSEEVENTF_LEFTUP;

	int nCount = LCTokenize(szArgs, tokens, 3, NULL);
	if(nCount == 1)
		szButton = szToken1;
	else if(nCount == 2)
	{
		szX = szToken1;
		szY = szToken2;
	}
	else if(nCount == 3)
	{
		szButton = szToken1;
		szX = szToken2;
		szY = szToken3;
	}

	POINT pointCursor;
	GetCursorPos(&pointCursor);
	nOldX = pointCursor.x;
	nOldY = pointCursor.y;

	if(szX)
	{
		nNewX = atoi(szX);
		nNewY = atoi(szY);
		if(nNewX < 0) nNewX += GetSystemMetrics(SM_CXSCREEN);
		if(nNewY < 0) nNewY += GetSystemMetrics(SM_CYSCREEN);

		bMove = TRUE;
	}

	if(szButton)
	{
		bDouble = (BOOL)strchr(szButton, 'd');
		if(strchr(szButton, 'm'))
		{
			DWORD dwDown = MOUSEEVENTF_MIDDLEDOWN;
			DWORD dwUp = MOUSEEVENTF_MIDDLEUP;
		}
		else if(strchr(szButton, 'r'))
		{
			DWORD dwDown = MOUSEEVENTF_RIGHTDOWN;
			DWORD dwUp = MOUSEEVENTF_RIGHTUP;
		}
	}

	while(TRUE)
	{
		if(bMove)
		{
			SetCursorPos(nNewX, nNewY);
			mouse_event(dwDown, NULL, NULL, NULL, NULL);
			SetCursorPos(nNewX, nNewY);
			mouse_event(dwUp, NULL, NULL, NULL, NULL);
			SetCursorPos(nOldX, nOldY);
		}
		else
		{
			mouse_event(dwDown, NULL, NULL, NULL, NULL);
			SetCursorPos(nOldX, nOldY);
			mouse_event(dwUp, NULL, NULL, NULL, NULL);
		}

		if(!bDouble) break;
		bDouble = FALSE;
	}
}
